@@ -1,20 +1,22 @@
 ###############################################################################################################################################
 #  Print method for class marssMLE. 
 ##############################################################################################################################################
-print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what="fit", form=NULL, silent=TRUE) {
-#First make sure specified equation form has a corresponding function to do the conversion to marssm object
+print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what="fit", form=NULL, silent=FALSE) {
+#First make sure specified equation form has a corresponding function to do the conversion to marssMODEL object
   return.obj=list()
   what.to.print = what
   orig.x=x
-  if(is.null(form)){ #allow user to printing using a different form than call$form
-    if(is.null(x[["call"]][["form"]])) form="marss" else form=x$call$form
+  if(is.null(form)){ #allow user to printing using a different form than x$model form
+    form = attr(x[["model"]], "form")
   }
-  print.fun = paste("print_",form,sep="")
+  print.fun = paste("print_",form[1],sep="")
   tmp=try(exists(print.fun,mode="function"),silent=TRUE)
   if(isTRUE(tmp)){
       #the print function can print or return an updated x to use for printing
+      #it changes the $par element to match $model instead of $marss
       x=eval(call(print.fun, x))
-    }
+    }else{ x = print_marss(x) } #if print_form is missing use print_marss
+
   if(class(x)=="marssMLE"){ #use x to print; if print.fun returns NULL, say, nothing happens
    if(identical(what.to.print,"fit")){
       ## all parameters were fixed
@@ -85,23 +87,23 @@ print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what
   cat("\n \n") 
   
   #set up the paramvector
-  paramvector=MARSSvectorizeparam(x)
+  paramvector=coef(orig.x,type="vector")
 
 	cmat = as.matrix(paramvector)
 	colnames(cmat) = "Estimate"
 	if(!is.null(x[["par.se"]])) {
-	  cmat = cbind(cmat, MARSSvectorizeparam(x,what="par.se"))
+	  cmat = cbind(cmat, coef(orig.x,type="vector",what="par.se"))
 	  colnames(cmat) = c("ML.Est", "Std.Err")
 	}
 
   if(!is.null(x[["par.lowCI"]]) & !is.null(x[["par.upCI"]])) {  
-	  cmat = cbind(cmat, MARSSvectorizeparam(x,what="par.lowCI"))
+	  cmat = cbind(cmat, coef(orig.x,type="vector",what="par.lowCI"))
 	  colnames(cmat)[dim(cmat)[2]]="low.CI"
-	  cmat = cbind(cmat, MARSSvectorizeparam(x,what="par.upCI"))
+	  cmat = cbind(cmat, coef(orig.x,type="vector",what="par.upCI"))
 	  colnames(cmat)[dim(cmat)[2]]="up.CI"  
 	}
 	if(!is.null(x[["par.bias"]])) {
-    par.bias = MARSSvectorizeparam(x,what="par.bias") 
+    par.bias = coef(orig.x,type="vector",what="par.bias") 
 	  cmat = cbind(cmat, par.bias)
 	  colnames(cmat)[dim(cmat)[2]]="Est.Bias"
     cmat = cbind(cmat, as.matrix(paramvector)+par.bias)
@@ -117,7 +119,13 @@ print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what
 	if(!is.null(x[["par.bias"]])){cat(paste("Bias calculated via",x$par.CI.method,"bootstrapping with",x$par.CI.nboot,"bootstraps. \n"))
   }
   
-  if(!is.null(x[["errors"]])) cat(x$errors)
+  if(!is.null(x[["errors"]])){
+    if(length(x$errors)>10){
+      cat(x$errors[1])
+      cat(paste(length(x$errors)-1," ",ifelse(grepl("warnings",x$errors[1]),"warnings","errors"),". First 10 shown.  Type cat(object$errors) to see the full list.\n ",sep=""))
+      cat(x$errors[2:11])
+    }else{ cat(x$errors) }
+  }
    
   }else{
     cat("marssMLE object $par element is NULL.  Parameters have not been estimated.\n")
@@ -140,14 +148,14 @@ print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what
     for(what in what.to.print){
     if(what=="fit") return.obj[[what]]=orig.x
     if(what=="model"){
-      print.fun=paste("print.model.",form,sep="")
-      if(!exists(print.fun)){ print(x$model)
-        }else{ eval(call(print.fun, x)) }
+      #note at beginning of this function x$model was replaced with the model
+      #in call$form form
+      print.marssMODEL(x$model)
       return.obj[[what]]=x$model
     }
     if(what=="par"){
       if(!silent){ cat("List of the estimated values in each parameter matrix\n"); print(x$par); cat("\n") }
-      return.obj[[what]]=x$par
+      return.obj[[what]]=coef(orig.x,type="vector")
     }
     if(what=="logLik"){
       if(!silent){ cat(paste("Log-likelihood: ",x$logLik,sep="")); cat("\n") }
@@ -157,51 +165,51 @@ print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what
       if(is.null(x[[what]])){        
         if(what!="par.bias"){ 
           cat(paste("No ",what," element on your marssMLE object.\nRunning MARSSparamCIs with method=hessian\n",sep=""))
-          x=MARSSparamCIs(x)
+          orig.x=MARSSparamCIs(orig.x)
         }else{
           cat("No par.bias element on your marssMLE object.\nRunning MARSSparamCIs with method=parametric to get parameter bias estimate.\n")
-          x=MARSSparamCIs(x,method="parametric")
+          orig.x=MARSSparamCIs(x,method="parametric")
         }
       } 
-      if(!silent){ cat(paste("Estimated ",what,":\n")); print(MARSSvectorizeparam(x,what=what)); cat("\n") }
-      return.obj[[what]]=x[[what]]
+      if(!silent){ cat(paste("Estimated ",what,":\n")); print(coef(orig.x,type="vector",what=what)); cat("\n") }
+      return.obj[[what]]=orig.x[[what]]
     }
     if(what=="paramvector"){
-      paramvector = MARSSvectorizeparam(x)
+      paramvector = coef(orig.x,type="vector")
       if(!silent){ cat("vector of the estimated parameter values\n"); print(paramvector); cat("\n") }
        return.obj[[what]]=paramvector
     }
     if(what=="xtT" | what=="states"){
       if(!silent){ cat("Smoothed state estimates\n"); print(x$states); cat("\n") }
-      return.obj[[what]]=x$states
+      return.obj[[what]]=orig.x$states
     }
     if(what=="data"){
-      if(!silent) print(x$model$data)
-      return.obj[[what]]=x$model$data
+      if(!silent) print(orig.x$model$data)
+      return.obj[[what]]=orig.x$model$data
     }
     if(what=="ytT"){
-      if(is.null(x[["ytT"]])){ ytT=MARSShatyt( x )$ytT }else{ ytT=x$ytT }
+      if(is.null(x[["ytT"]])){ ytT=MARSShatyt( orig.x )$ytT }else{ ytT=x$ytT }
       if(!silent){ cat("Expected value of Y(t) conditioned on all the data.  If data are present ytT=y.\n"); print(ytT); cat("\n") }
       return.obj[[what]]=ytT
     }
     if(what=="states.se"){
-      if(!silent){ cat("Smoothed state standard errors.\n"); print(x$states.se); cat("\n") }
-      return.obj[[what]]=x$states.se
+      if(!silent){ cat("Smoothed state standard errors.\n"); print(orig.x$states.se); cat("\n") }
+      return.obj[[what]]=orig.x$states.se
     }
     if(what=="model.residuals"){
-      res = residuals(x)$model.residuals
+      res = residuals(orig.x)$model.residuals
       if(!silent){ cat("Smoothed model residuals.\n"); print(res); cat("\n") }
       return.obj[[what]]=res
     }
     if(what=="state.residuals"){
-      res = residuals(x)$state.residuals
+      res = residuals(orig.x)$state.residuals
       if(!silent){ cat("Smoothed state residuals.\n"); print(res); cat("\n") }
       return.obj[[what]]=res
     }
     if(what=="kfs"){
-      kf = MARSSkf(x)
+      kf = MARSSkf(orig.x)
       if(x$fun.kf=="MARSSkfas"){
-        tmp=MARSSkfss(x)   #MARSSkfas doesn't return Innov or Sigma
+        tmp=MARSSkfss(orig.x)   #MARSSkfas doesn't return Innov or Sigma
         kf$Innov=tmp$Innov
         kf$Sigma=tmp$Sigma
       }
@@ -209,28 +217,28 @@ print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what
       return.obj[[what]]=kf
     }
     if(what=="Ey"){
-      Ey = MARSShatyt(x)  #MARSSkf since it outputs more info
+      Ey = MARSShatyt(orig.x)  #MARSSkf since it outputs more info
       if(!silent){ cat("Expectations involving y conditioned on all the data.  See ?MARSShatyt. \n Output is verbose.  Assign print output and look at assigned object.\n"); cat("\n") }
       return.obj[[what]]=Ey
     }
     if(what=="states.cis"){
       if(!silent) cat("Approximate 95% confidence intervals for the states using 1.95*states.se.\n\n")
-      rpt.list=list(up95CI=x$states+qnorm(0.975)*x$states.se,est=x$states, low95CI=x$states-qnorm(0.975)*x$states.se)
+      rpt.list=list(up95CI=orig.x$states+qnorm(0.975)*orig.x$states.se,est=orig.x$states, low95CI=orig.x$states-qnorm(0.975)*orig.x$states.se)
       if(!silent){ print(rpt.list); ; cat("\n") }
       return.obj[[what]]=rpt.list
     }
     if(what=="start" | what=="inits"){
       if(!silent) cat("Initial parameter values used for optimization.\n\n")
-      if(!silent){ print(MARSSvectorizeparam(x,what="start")); ; cat("\n") }
-      return.obj[[what]]=x$start
+      if(!silent){ print(coef(orig.x,type="vector",what="start")); ; cat("\n") }
+      return.obj[[what]]=coef(orig.x,type="vector",what="start")
     }
     fixed=x$model$fixed
     free=x$model$free
     #if there is info on the parameter dimensions in the model list, use it
-    if(!is.null(x[["model"]][["model.dims"]])){ par.dims=x$model$model.dims }else{ par.dim=NULL }
+    par.dims=attr(x$model, "model.dims")
     if(what %in% names(fixed)){
       par.dim=par.dims[[what]]
-      the.par = parmat(x,what,t=1:max(dim(fixed[[what]])[3],dim(free[[what]])[3]), dims=par.dim)[[what]]
+      the.par = parmat(x,what,t=1:max(dim(fixed[[what]])[3],dim(free[[what]])[3]), dims=par.dim, model.loc="model")[[what]]
       if(!silent){ cat(paste("Parameter matrix ",what,"\n", sep="")); print(the.par); cat("\n") }
       return.obj[[what]]=the.par
     }
@@ -246,4 +254,6 @@ print.marssMLE <- function (x, digits = max(3, getOption("digits")-4), ..., what
   } #x is type marssMLE  
  }  #end of print.marssMLE
  
- print_marss = function(x){ return(x) }
\ No newline at end of file
+ print_marss = function(x){ 
+   x$model = x$marss
+   return(x) }
\ No newline at end of file
